
'''

wlnad.py
==========

Desc: run wlnm agent deamon as SimpleXMLRPCServer


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


from SimpleXMLRPCServer import SimpleXMLRPCServer
from daemon import Daemon
from os.path import expanduser
import os
import sys


import agent
import util
import database

def list_contents(dir_name):
	return os.listdir(dir_name)

def searchAll():
	return agent.searchAll()

def nmstart(wls_home):
	return agent.nmstart(wls_home)

def startAdmin(domainhome):
	return agent.startAdmin(domainhome)

def startServer(args):
	return agent.startServer(args)

def stopServer(args):
	return agent.stopServer(args)

def listprocess():
	return agent.listprocess()

def kill(args):
	return agent.kill(args)

wlnad_pid_file = "%s/.wlnm/wlnad.pid" % expanduser("~")

database.creatProfileDir()

class AgentDaemon(Daemon):
	
	
	def __init__(self,pidfile, port,stdin=os.devnull, stdout=os.devnull, stderr=os.devnull, home_dir='.', umask=022, verbose=1):
		wlnad_out_file = "%s/.wlnm/wlnad.out" % expanduser("~")
		self.stdin = stdin
		self.stdout = wlnad_out_file
		self.stderr = wlnad_out_file
		self.pidfile = pidfile
		self.home_dir = home_dir
		self.verbose = verbose
		self.umask = umask
		self.daemon_alive = True
		self.port = port


        def start(self):

		# Check port see if it is open
		if util.checkport(self.port) :
			message = "port %s is occupied by other process\n"
                        sys.stderr.write(message % self.port)
                        sys.exit(1)
		super(AgentDaemon, self).start()
	
	def run(self):
                server = SimpleXMLRPCServer(('0.0.0.0', self.port), logRequests=True)
		server.register_function(list_contents)
		server.register_function(searchAll)
		server.register_function(nmstart)
		server.register_function(startAdmin)
		server.register_function(startServer)
		server.register_function(stopServer)
		server.register_function(listprocess)
		server.register_function(kill)
		

		try:
			#print 'Use Control-C to exit'
			server.serve_forever()
		except KeyboardInterrupt:
			print 'Exiting'

	



def runDaemon(iport):
	print "Start wlnm agent daemon ."
	daemon = AgentDaemon(pidfile=wlnad_pid_file,port=iport)
	daemon.start()
	


def stopDaemon():
	
	if os.path.isfile(wlnad_pid_file):
	    with open(wlnad_pid_file) as f:
		pids = f.readlines()
	    try:
		os.kill(int(pids[0]),9)	#force kill
		print "wlnm agent process %s is stopped ." % int(pids[0])
	    except :
		print "wlnm agent process %s was killed before ." % int(pids[0])
	    finally :
		os.remove(wlnad_pid_file)
	else :
	    print "wlnm agent is not running , no process can be stopped."
	
	print

#runDaemon(9099)
#stopDaemon()



'''

inputer.py
==========

Desc: Helper module for command which user typed


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import readline


import util

from wlnsc import ServerClient


#server_inst = ServerClient(host="localhost",port="9099")
#server_inst.connect()

class remoteCmds():
	def __init__(self,server):
		self.server = server


		self.cmddict={'init':self.server.initMachine,
			'ls':self.server.show,
			'lswls':self.server.showWLS,
			'lsd':self.server.showDomains,
			'lsm':self.server.showMachines,
			'nmstart':self.server.nmstart,
			'help':util.help,
			'startadmin':self.server.startAdmin,
			'start':self.server.startServer,
			'stop':self.server.stopServer,
			'lsp':self.server.listprocess,
			'kill':self.server.kill,
			'use':self.server.setMachine,
			'disconnect':self.server.disconnect,
			'quit':sys.exit
		}

		self.cmds=self.cmddict.keys()

	def runcmd(self,command,args):
	
		if command not in self.cmds:
			print 'Invilad command! Press Tab key or use help command list all avaiable command.'
			print ''
		else:
			if not args :
				if command == "ls" :
					self.cmddict[command]([])
				else :
					self.cmddict[command]()
			else:
				self.cmddict[command](args)



	def validcmd(self,command,args):
		#print self.server.machine
		#print command
		if command in ["ls","lsd" ,"lsp" ,"lswls" ,"disconnect","nmstart" ,"startadmin" ,"start" , "stop" , "kill"] and not self.server.machine:
			print "You have to connect to one machine to use %s" % command
			return
		if command in ["lsd" ,"lsp" ,"lswls" , "lsm" , "quit","disconnect","help"] and args :
			print "no arguments for command %s" % command
			return
		if command in ["init","nmstart" ,"startadmin" ,"start" , "stop" , "kill","use"] and not args :
			print "arguments needed run command %s" % command
			return
		self.runcmd(command,args)




cmds=["init","ls","lsd","lsm" ,"lsp" ,"lswls" ,"use","disconnect","nmstart" ,"startadmin" ,"start" , "stop" , "kill","quit","help"]

def complete(text, state):
    for cmd in cmds:
		if cmd.startswith(text):
		    if not state:
			return cmd
		    else:
			state -= 1


#!/usr/bin/env python

import sys
import argparse

class ThrowingArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise Exception(message)

def main():

	try:
		parser = ThrowingArgumentParser(prog='wlnm',epilog="Weblogic node master , a command tool to manage weblogic easily")
		#parser = argparse.ArgumentParser
		group = parser.add_mutually_exclusive_group()
		parser.add_argument('command', choices=['start', 'stop', 'connect'], help='start|stop|connect')
		group.add_argument('-s',  dest='server', default=False, help='Run weblogic node server')
		group.add_argument('-a', '--agent', action="store_true", dest='agent', help='Run weblogic node agent')
		parser.add_argument('-p', '--port', type =int ,help='Listen port',required=True)
		args = parser.parse_args()

	except SystemExit:
		pass
	except  Exception ,e:
		print
		print e
		print
		parser.print_help()
		
	
	

if __name__ == "__main__":
    main()


'''

webserver.py
==========

Desc: web server of wlnm


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 


'''

import os
from os import environ as env
import sys
import bottle
from bottle import route, run, static_file, redirect, request, error
from beaker.middleware import SessionMiddleware
import server ,util,database
from os.path import expanduser

from daemon import Daemon

bottle.debug(True)

WEBPATH = os.path.dirname(os.path.abspath(__file__))
webroot = '%s/app' % WEBPATH


# session set up
app = bottle.default_app()
session_opts = {
	#'session.type': 'file',
	'session.cookie_expires': 30000,
	#'session.data_dir': './session',
	'session.auto': True,
	'session.type': 'memory',
	#'session.key' : 'wlnm',
	#session.secret = 0cb243f53ad865a0f70099c0414ffe9cfcfe03ac
}
app = SessionMiddleware(app, session_opts)


# static files (js,css,templates) set up

@route('/lib/:path#.+#')
def server_static(path):
    return static_file(path, root='%s/lib/' % webroot)
'''
@route('/scenarios.js')
def scenario():
    return static_file('scenarios.js',root='./test/e2e/')
'''
@route('/favicon.ico')
def favicon():
    return static_file('favicon.ico', root=webroot)

@route('/js/:path#.+#')
def server_static(path):
    return static_file(path, root='%s/js/' % webroot)

@route('/css/:path#.+#')
def server_static(path):
    return static_file(path, root='%s/css/' % webroot)

@route('/img/:path#.+#')
def server_static(path):
    return static_file(path, root='%s/img/' % webroot)

@route('/templates/:path#.+#')
def server_static(path):
    return static_file(path, root='%s/templates/' % webroot)

# route set up
@route('/settings',method='POST')
def save_settings():
    s = request.environ.get('beaker.session')
    s['wlnm'] = request.POST.get('machine').strip()
    s.save()
    redirect ('/')


@route('/command/:id',method='GET')
def get_commands(id):
     args = id.split(" ")[1:]
     cmd = id.split(" ")[0]
     result = run_command(cmd,args)
     return result


@route('/')
@route('/index.html')
def index():
    raise static_file('index.html', root=webroot)

'''
@route('/tests')
def tests():
    raise static_file('runner.html', root='./test/e2e/')
'''


@error(404)
def mistake404(code):
    return static_file('404.html', root=webroot)


def runServer(port):
    bottle.run(app=app,host='0.0.0.0', port=port)

def saveMachine(machine):
	s = request.environ.get('beaker.session')

	s['machine'] = machine

	s.save()

	return 'Machine %s is set!' % machine

def ifSession(key):
	try :
		s = request.environ.get('beaker.session')
		print s
		if key in s.keys():
			return True
		else :
			return False
	except :
		return False


def run_command(cmd,args=[]):
	

	machine =''
	
	if ifSession('machine'):
		s = request.environ.get('beaker.session')
		machine = s['machine']
		print machine


	cmddict={'init':server.initMachine,
		'ls':server.show,
		'lswls':server.showWLS,
		'lsd':server.showDomains,
		'lsm':server.showMachines,
		'nmstart':server.nmstart,
		'help':util.help,
		'startadmin':server.startAdmin,
		'start':server.startServer,
		'stop':server.stopServer,
		'lsp':server.listprocess,
		'kill':server.kill,
		'use':server.useMachine,
		# 'disconnect':server.disconnect,
		}
	
	if cmd not in cmddict.keys():
		return 'invalid command , use help to check commands.'

	if cmd not in ['help','lsm','use','init'] and not machine :
		return 'you must connect one machine to use %s' % cmd
		
	
	if cmd == 'help' :
		mystdout,old_stdout = util.redirectStdout()
		cmddict[cmd]()
		util.finishRedirectStdout(old_stdout)
		return mystdout.getvalue()
	elif cmd == 'use' :
		try:
			result = cmddict[cmd](args) 
			port = int(result)
			saveMachine(args[0])
			return '%s is ready to use . ' % args[0]
			
		except:
			return result
	elif cmd == 'init' :
		return cmddict[cmd](args[0],args[1])
	elif cmd == 'lsm' :
		return util.decode_output(cmddict[cmd]())
	elif cmd in ['lswls','lsp','lsd']:
		return util.decode_output(cmddict[cmd](machine))
	else :
		#args.insert(0,cmd)
		args.insert(0,machine)
		print args
		return util.decode_output(cmddict[cmd](args))

	

# start application
#database.checkdb()
#bottle.run(app=app,host='0.0.0.0', port=argv[1])

wlnws_pid_file = "%s/.wlnm/wlnws.pid" % expanduser("~")

database.creatProfileDir()

class webserverDaemon(Daemon):

	def __init__(self,pidfile, port,stdin=os.devnull, stdout=os.devnull, stderr=os.devnull, home_dir='.', umask=022, verbose=1):
		wlnws_out_file = "%s/.wlnm/wlnws.out" % expanduser("~")
		self.stdin = stdin
		self.stdout = wlnws_out_file
		self.stderr = wlnws_out_file
		self.pidfile = pidfile
		self.home_dir = home_dir
		self.verbose = verbose
		self.umask = umask
		self.daemon_alive = True
		self.port = port
	

	def start(self):

		# Check port see if it is open
		if util.checkport(self.port) :
			message = "port %s is occupied by other process\n"
                        sys.stderr.write(message % self.port)
                        sys.exit(1)
		super(webserverDaemon, self).start()

	def run(self):
		try:
			database.checkdb()
			bottle.run(app=app,host='0.0.0.0', port =self.port)
		except Eception,e:
			print e
			sys.exit(1)

def runDaemon(iport):
	print "Start wlnm web server daemon ."
	daemon = webserverDaemon(pidfile=wlnws_pid_file,port=iport)
	daemon.start()
	


def stopDaemon():
	
	if os.path.isfile(wlnws_pid_file):
	    with open(wlnws_pid_file) as f:
		pids = f.readlines()
	    try:
		os.kill(int(pids[0]),9)	#force kill
		print "wlnm web server process %s is stopped ." % int(pids[0])
	    except :
		print "wlnm web server process %s was killed before ." % int(pids[0])
	    finally :
		os.remove(wlnws_pid_file)
	else :
	    print "wlnm server is not running , no process can be stopped."
	
	print


'''

database.py
==========

Desc: Functions for db persistense


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''
import shelve
import os
from os.path import expanduser

wlnm_data_file = "%s/.wlnm/wlnm.dat" % expanduser("~")


def creatProfileDir():
	pdir = "%s/.wlnm" % expanduser("~")
	if not os.path.exists(pdir):
		os.makedirs(pdir,0700)


def checkdb():
    if not os.path.isfile(wlnm_data_file):
	creatProfileDir()
        initDB()

def initDB():
	db = shelve.open(wlnm_data_file,"c")
	db['wlnm'] = {}
	db.close()

def loadAllDB():
	db = shelve.open(wlnm_data_file, "r")
	wlnmDB = db['wlnm']
	machines = wlnmDB.keys()
	return ((wlnmDB,machines))

def loadDB(machine,wlss=False,domain=False,server=False):
	
	db = shelve.open(wlnm_data_file, "r")
	wlsDB = None
	wls = None
	domainsDB = None
	domains = None
	serversDB = None
	servers = None

	if wlss :
		wlsDB = db['wlnm'][machine]['wls']
		wls = wlsDB.keys()
	if domain:
		domainsDB = db['wlnm'][machine]['domains']
		domains = domainsDB.keys()
	if server :
		serversDB = db['wlnm'][machine]['servers']
		servers = serversDB.keys()
	db.close()
	return ([wlsDB,wls],[domainsDB,domains],[serversDB,servers])


def saveDB(machine,mac=False,wlss=False,domain=False,server=False,data=None):
	#print wlnm_data_file
	db = shelve.open(wlnm_data_file,"c")
	alldata = db['wlnm']
	if mac :
		port = data[0]
		alldata[machine] = {"agentport":port,"wls":{},"domains":{},"servers":{}}

	if wlss :
		wlsHome,version,nmPort = data[1]
		alldata[machine]['wls']['wls'+version] = {"home":wlsHome,"version":version,"nmport":nmPort}
		#print "wls%s has been store in local db!" % version

	if domain :
		domainName,domainhome,version,adminurl,user,pwd = data[2]

		alldata[machine]['domains'][domainName] = {"home":domainhome,"version":version,"adminurl":adminurl,"user":user,"pwd":pwd}
		#print "%s has been store in local db!" % domainName
	
	if server :
		name,host,port,type,domain = data[3]

		_key = "%s.%s" % (name,domain)
		alldata[machine]['servers'][_key] = {"name":name,"host":host,"port":port,"type":type,"domain":domain}
		
		#print "%s has been store in local db!" % _key

	db['wlnm']=alldata

	db.close()





'''

wlnsd.py
==========

Desc: run wlnm server deamon as SimpleXMLRPCServer


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


from SimpleXMLRPCServer import SimpleXMLRPCServer

from os.path import expanduser
import os
import sys

import server
import util
from daemon import Daemon
import database

def list_contents(dir_name):
	return os.listdir(dir_name)

def initMachine(machine, port):
	return server.initMachine(machine, port)

def showMachines():
	return server.showMachines()

def useMachine(args):
	return server.useMachine(args)

def show(args):
	return server.show(args)

def showWLS(machine):
	return server.showWLS(machine)

def showDomains(machine):
	return server.showDomains(machine)

def nmstart(args):
	return server.nmstart(args)

def startAdmin(args):
	return server.startAdmin(args)

def startServer(args):
	return server.startServer(args)

def stopServer(args):
	return server.stopServer(args)

def listprocess(machine):
	return server.listprocess(machine)

def kill(args):
	return server.kill(args)

wlnsd_pid_file = "%s/.wlnm/wlnsd.pid" % expanduser("~")

database.creatProfileDir()

class ServerDaemon(Daemon):
	
	
	def __init__(self,pidfile, port,stdin=os.devnull, stdout=os.devnull, stderr=os.devnull, home_dir='.', umask=022, verbose=1):
		wlnsd_out_file = "%s/.wlnm/wlnsd.out" % expanduser("~")
		self.stdin = stdin
		self.stdout = wlnsd_out_file
		self.stderr = wlnsd_out_file
		self.pidfile = pidfile
		self.home_dir = home_dir
		self.verbose = verbose
		self.umask = umask
		self.daemon_alive = True
		self.port = port

	def start(self):

		# Check port see if it is open
		if util.checkport(self.port) :
			message = "port %s is occupied by other process\n"
                        sys.stderr.write(message % self.port)
                        sys.exit(1)
		super(ServerDaemon, self).start()

        def run(self):
		database.checkdb()
                server = SimpleXMLRPCServer(('0.0.0.0', self.port), logRequests=True)
		server.register_function(list_contents)
		server.register_function(initMachine)
		server.register_function(useMachine)
		server.register_function(showMachines)
		server.register_function(show)
		server.register_function(showWLS)
		server.register_function(showDomains)
		server.register_function(nmstart)
		server.register_function(startAdmin)
		server.register_function(startServer)
		server.register_function(stopServer)
		server.register_function(listprocess)
		server.register_function(kill)
		print "wlnm server daemon is running ."
		try:
			#print 'Use Control-C to exit'
			server.serve_forever()
		except KeyboardInterrupt:
			print 'Exiting'

	



def runDaemon(iport):
	print "Start wlnm server daemon ."
	daemon = ServerDaemon(pidfile=wlnsd_pid_file,port=iport)
	daemon.start()
	


def stopDaemon():
	
	if os.path.isfile(wlnsd_pid_file):
	    with open(wlnsd_pid_file) as f:
		pids = f.readlines()
	    try:
		os.kill(int(pids[0]),9)	#force kill
		print "wlnm server process %s is stopped ." % int(pids[0])
	    except :
		print "wlnm server process %s was killed before ." % int(pids[0])
	    finally :
		os.remove(wlnsd_pid_file)
	else :
	    print "wlnm server is not running , no process can be stopped."
	
	print

#runDaemon(9099)
#stopDaemon()



#!/usr/bin/env python

'''

wlnm.py
==========

Desc: Entry point of weblogic node master 


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import argparse

import os
import logging

import socket
import readline

import inputer,search,util

from wlnsc import ServerClient
from inputer import remoteCmds

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s')

 

def run(server_inst):

    print "Weblogic Node Master"
    print ""
    print "Type help to load help page ."
    print ""

    command =''
    readline.parse_and_bind("tab: complete")
    readline.set_completer(inputer.complete)
    mycmd = remoteCmds(server_inst)
    while True :
        
	
	if server_inst.machine :
		i_input = raw_input ('wlnm(%s)>>' % server_inst.machine)
	else:
		i_input = raw_input ('wlnm>>')
	if not i_input : continue
	command = i_input.split()[0]
	args=i_input.split()[1:]
	
	
	mycmd.validcmd(command,args)
	
    print "Bye"


class MyArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise Exception(message)

def main():
	#print "----"
	args = None
	try:
		parser = MyArgumentParser(prog='wlnm')
		parser.add_argument('-s', '--server' ,dest ='host', help='Hostname or IP of wlnm server,default is localhost',default="localhost")
		parser.add_argument('-p', '--port', type =int ,help='Listen port,default is 9099',default="9099")

		args = parser.parse_args()
		

	except SystemExit:
		pass
	except  Exception ,e:
		
		parser.print_help()

		print
		print "Arguments Error"
		print "---------------"
		print e
		print

		sys.exit()
	
	if not args :  sys.exit()

	if not util.checkport(args.port,host=args.host) :
			
		print " wlnm server at %s:%s is not reachable\n" % (args.host,args.port)
		sys.exit()
	

	try :
		server_inst = ServerClient(host=args.host,port=args.port)
		server_inst.connect()
		
	except Exception,e:
		print "wlnm server %s:%s is not able to connect." % (args.host,args.port)
		print
		raise e
		sys.exit()
	run(server_inst)

	

if __name__ == "__main__":
    main()




'''

wlnac.py
==========

Desc:  wlnm client to connect wlnode agent


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import xmlrpclib
import util


class agentclient():

	def __init__(self,host,port):

		self.host = host
		self.port = port
		self.conn = None


	def connect(self):
		try:
			
			self.conn = xmlrpclib.ServerProxy('http://%s:%s' % (self.host,self.port))
			self.conn.list_contents('.')

		except:
			raise Exception('not able to connect to http://%s:%s ,make sure port and host is corrected' % (self.host,self.port))


	def searchAll(self):
		return self.conn.searchAll()

	def nmstart(self,args):
		print util.decode_output(self.conn.nmstart(args))

	def startAdmin(self,args):
		print util.decode_output(self.conn.startAdmin(args))

	def startServer(self,args):
		print util.decode_output(self.conn.startServer(args))

	def stopServer(self,args):
		print util.decode_output(self.conn.stopServer(args))

	def listprocess(self):
		print util.decode_output(self.conn.listprocess())

	def kill(self,args):
		print util.decode_output(self.conn.kill(args))

	

#connect(["localhost","9099"])
#print proxy.list_contents('/tmp')


#!/usr/bin/env python

'''

wlnm.py
==========

Desc: Entry point of weblogic node agent 


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import argparse
import time

import wlnad

class MyArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise Exception(message)


def main():
	
	args = None
	try:
		parser = MyArgumentParser(prog='wlna')
		parser.add_argument('command', choices=['start', 'stop','restart'], help='start|stop|restart wlnm server process')
		parser.add_argument('-p', '--port', type =int ,help='Listen port,default port is 9098',default="9098")
		args = parser.parse_args()
		if args.command == "start" and not args.port :
			raise Exception("Port is not provied when you try to start process")

	except SystemExit:
		pass
	except  Exception ,e:
		
		parser.print_help()

		print
		print "Arguments Error"
		print "---------------"
		print e
		print

		sys.exit()
	
	if not args :  sys.exit()
	if args.command == "start" :
		wlnad.runDaemon(int(args.port))

	elif args.command == "stop":
		wlnad.stopDaemon()
	else:
		
		wlnad.stopDaemon()
		time.sleep(1)
		wlnad.runDaemon(int(args.port))
	
if __name__ == "__main__":
    main()


'''

starter.py
==========

Desc: AdminServer and node manager process admin command including start and kill


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


import subprocess
import psutil

import search
import util



def startAdmin(domainhome):
	
	proc = subprocess.Popen('nohup %s/startWebLogic.sh 1>>/dev/null 2>&1 &' % domainhome, shell=True, stdout=subprocess.PIPE)
	#out, err = proc.communicate()
	#print out
	print "Admin server process starting "

def kill(args):
	arg = args[0]
	if util.RepresentsInt(arg):
		killbyport(arg)
	else:
		killbyname(arg)

def killbyname(name):
	for p in psutil.process_iter():
			for subcmd in p.cmdline :
				if '-Dweblogic.Name=' in subcmd and name == subcmd.split("=")[-1]:
					p.kill()
					print "%s is stopped!" % name

def killbyport(port):
	 pid = ''
	 
	 cmd_str = "netstat -pan 2>/dev/null|grep %s|grep LISTEN|tr -s ' ' '/' |cut -f7 -d '/'|tail -1" % port
	 p = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
	 pid = p.communicate()[0].strip()

	 if util.RepresentsInt(pid):
		 proc = psutil.Process(int(pid))
		 proc.kill()
		 print "process listen on %s is killed ." % port
	 else:
		print "no process listen on %s!" % port

def nmstart(wls_home):
	cmd_str = 'nohup %s/server/bin/startNodeManager.sh 1>/dev/null 2>&1 &' % wls_home
	runcmd(cmd_str)

def runcmd(cmd_str):
	#print cmd_str
	proc = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
	#while proc.poll() is None:
	    #output = proc.stdout.readline()
	    #print output
	print "Nodemanager started."




'''

agent.py
==========

Desc: agent facade ,  remote call interface


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 


'''

import traceback

import util
import search
import starter
import monitor
import wls

def searchAll():
	
	return search.searchAll()
	


def nmstart(wls_home):
	mystdout,old_stdout = util.redirectStdout()
	try:
		starter.nmstart(wls_home)
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startAdmin(domainhome):
	mystdout,old_stdout = util.redirectStdout()
	try:
		starter.startAdmin(domainhome)
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startServer(args):
	mystdout,old_stdout = util.redirectStdout()
	try:
		wls.startServer(args)
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def stopServer(args):
	mystdout,old_stdout = util.redirectStdout()
	try:
		wls.stopServer(args)
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def listprocess():
	mystdout,old_stdout = util.redirectStdout()
	try:
		monitor.listprocess()
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def kill(args):
	mystdout,old_stdout = util.redirectStdout()
	try:
		starter.kill(args)
	except :
		traceback.print_exc()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())






'''

wlnsc.py
==========

Desc:  wlnm client to connect wlnode server


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import xmlrpclib
import util
import inputer
import sys

class ServerClient():

	def __init__(self,host,port):

		self.host = host
		self.port = port
		self.conn = None
		self.machine = None
		self.aPort = port

	def connect(self):
		try:
			self.conn = xmlrpclib.ServerProxy('http://%s:%s' % (self.host,self.port))
			
			self.conn.list_contents('.')
			

		except:
			#print 'not able to connect to http://%s:%s ,make sure port and host is corrected' % (self.host,self.port)
			message = 'Having trouble to communicate with wlnm server %s:%s ,you may need to restart it \n'
                        sys.stderr.write(message % (self.host,self.port))
                        sys.exit(1)

	def setMachine(self,args):
		self.machine = args[0]
		
		result =self.conn.useMachine(args)
		try:
			self.aPort = int(result)
			
		except:
			print result


	def disconnect(self):
		self.machine = None
		self.aPort = None
	
	def showMachines(self):
		print util.decode_output(self.conn.showMachines())
	
	def initMachine(self,args):
		print  self.conn.initMachine(args[0], args[1])

	def show(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.show(args))

	def showWLS(self):
		print util.decode_output(self.conn.showWLS(self.machine))

	def showDomains(self):
		print util.decode_output(self.conn.showDomains(self.machine))

	def nmstart(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.nmstart(args))

	def startAdmin(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.startAdmin(args))

	def startServer(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.startServer(args))

	def stopServer(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.stopServer(args))

	def listprocess(self):
		print util.decode_output(self.conn.listprocess(self.machine ))

	def kill(self,args):
		args.insert(0,self.machine)
		print util.decode_output(self.conn.kill(args))

	


#s = ServerClient(host="localhost",port="8088")
#s.connect()


'''

server.py
==========

Desc: server facade ,  remote call interface


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 


'''

import util
import search
from wlnac import agentclient
import traceback

def showMachines():

	mystdout,old_stdout = util.redirectStdout()
	search.showMachines()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def initMachine(machine, port):

	try:
		agent = agentclient(host=machine,port=port)
		agent.connect()
		wlss,servers,domains = agent.searchAll()
		search.saveMachine(machine,port)
		
		for wls in wlss :
			search.saveWLS(machine,*wls)
		for server in servers :
			search.saveServer(machine,*server)
		for domain in domains :
			search.saveDomain(machine,*domain)

		return "%s initialise successfully." % machine
	except:
		traceback.print_exc()
		return "%s initialise failed." % machine

def useMachine(args):
	machine = args[0]
	try:
		
		port = search.getMachineAgentPort(machine)
		if  not port:
			return "%s is not initiliased , please use init commmand first. " % machine
		else :
			return port
	except:
		traceback.print_exc()
		return "wlnm server fail, check server log."

def show(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	search.show(machine,args[1:])
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def showWLS(machine):
	mystdout,old_stdout = util.redirectStdout()
	search.showWLS(machine)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def showDomains(machine):
	mystdout,old_stdout = util.redirectStdout()
	search.showDomains(machine)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def nmstart(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	nmport = args[1]
	port = search.getMachineAgentPort(machine)
	try:
		wls_home = search.getwlsHomeByNmport(machine,nmport)
		
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.nmstart(wls_home)
		print ""
	except Exception as e:
		traceback.print_exc()
		print "nmstart invoke failed on %s " % machine
		print e


	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startAdmin(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	domain = args[1]
	domainhome = ''
	port = search.getMachineAgentPort(machine)
	try:
		domainhome = search.getDomainHome(machine,domain)
		if not domainhome :
			raise Exception("%s is not resigtered !" % domain)
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.startAdmin(domainhome)
		print ""
	except Exception as e:
		traceback.print_exc()
		print "startAdmin invoke failed on %s " % machine
		print e

	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startServer(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	servername = args[1]
	port = search.getMachineAgentPort(machine)
	try:
		(adminurl,wlsHome,user,pwd) = search.getwls(machine,servername)
		if not wlsHome :
			raise Exception("could not find %s , make sure you spell correct" % name)
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.startServer((adminurl,wlsHome,user,pwd,servername))
		print ""
	except Exception as e:
		traceback.print_exc()
		print "startServers invoke failed on %s " % machine
		print e

	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def stopServer(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	servername = args[1]
	port = search.getMachineAgentPort(machine)
	try:
		(adminurl,wlsHome,user,pwd) = search.getwls(machine,servername)
		if not wlsHome :
			raise Exception("could not find %s , make sure you spell correct" % name)
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.stopServer((adminurl,wlsHome,user,pwd,servername))
		print ""
	except Exception as e:
		traceback.print_exc()
		print "stopServers invoke failed on %s " % machine
		print e

	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def listprocess(machine):
	mystdout,old_stdout = util.redirectStdout()
	port = search.getMachineAgentPort(machine)
	try:
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.listprocess()
		print ""
	except:
		traceback.print_exc()
		print "listprocess invoke failed on %s " % machine

	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def kill(args):
	mystdout,old_stdout = util.redirectStdout()
	machine = args[0]
	port = search.getMachineAgentPort(machine)
	try:
		# check arg
		
		if util.RepresentsInt(args[1]):
			if not search.isWlsPort(machine,args[1]) :
				raise Exception ("%s is not a valid weblogic port" % args[1])
			
		
		agent = agentclient(host=machine,port=port)
		agent.connect()
		agent.kill(args[1:])
		
		print ""
	except Exception as e:
		traceback.print_exc()
		print "kill invoke failed on %s " % machine
		print e

	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())


''' not for remote call '''
'''
def getAgent(host,port):
	try:
		agent = agentclient(host=host,port=port)
		agent.connect()

		return agent
	except:
		return
'''


